#include "image.h"
#include "tonc_memmap.h"
#include <string.h>

image_tileset_t* image_tileset_from_file(filesystem_file_t* file) {
	return (image_tileset_t*)filesystem_get_file_data(file);
}

const void* image_get_tiles(image_tileset_t* tileset) {
	return (const void*)(tileset + 1);
}

const void* image_get_palette(image_tileset_t* tileset) {
	const u8* palette = (u8*)(tileset + 1);
	palette += tileset->imageLength;

	return palette;
}

const TILE4* image_get_tiles4bpp(image_tileset_t* tileset) {
	if (tileset->format != IMAGE_4BPP) {
		return NULL;
	}

	return (const TILE4*)image_get_tiles(tileset);
}

const TILE8* image_get_tiles8bpp(image_tileset_t* tileset) {
	if (tileset->format != IMAGE_8BPP) {
		return NULL;
	}

	return (const TILE8*)image_get_tiles(tileset);
}

const u32* image_get_tilesraw(image_tileset_t* tileset) {
	if (tileset->format != IMAGE_RAW) {
		return NULL;
	}

	return (const u32*)image_get_tiles(tileset);
}

void image_load_tileset(image_tileset_t* tileset, u32 charblock) {

	void* charblockStart;
	if (tileset->format == IMAGE_8BPP) {
		charblockStart = tile8_mem[charblock];
	} else {
		charblockStart = tile_mem[charblock];
	}

	// Copy the tiles into the charblock

	const void* tilesetStart = image_get_tiles(tileset);
	memcpy16(charblockStart, tilesetStart, tileset->imageLength / 2);

}

void image_load_palette256(image_tileset_t* tileset, palette_t palette) {
	const COLOR* paletteStart = image_get_palette(tileset);	
	const COLOR* paletteDst;

	if (palette == PALETTE_BG) {
		paletteDst = pal_bg_mem;
	} else {
		paletteDst = pal_obj_mem;
	}

	memcpy16((void*)paletteDst, (void*)paletteStart, tileset->paletteLength / 2);
}

void image_load_spriteset(image_tileset_t* tileset) {

	void* charblockStart;
	if (tileset->format == IMAGE_8BPP || tileset->format == IMAGE_8BPP2D) {
		charblockStart = tile8_mem_obj[0];
	} else {
		charblockStart = tile_mem_obj[0];
	}

	// Copy the tiles into the charblock

	const void* tilesetStart = image_get_tiles(tileset);
	memcpy16(charblockStart, tilesetStart, tileset->imageLength / 2);

}
