#pragma once

#include <stdint.h>
#include "tonc_memmap.h"

/**
 * Input functions
 *
 * @author Casey
 */

// Global key state

extern u16 gCurrentKeyState;
extern u16 gPreviousKeyState;

#define KEY_A        0x0001
#define KEY_B        0x0002
#define KEY_SELECT   0x0004
#define KEY_START    0x0008
#define KEY_RIGHT    0x0010
#define KEY_LEFT     0x0020
#define KEY_UP       0x0040
#define KEY_DOWN     0x0080
#define KEY_R        0x0100
#define KEY_L        0x0200
#define KEY_MASK     0x03FF

/**
 * Poll the key state
 * This is placed in IWRAM because it is expected to be called from
 * an ISR, which should be running from IWRAM.
 */
IWRAM_CODE static inline void input_poll() {
  gPreviousKeyState = gCurrentKeyState;

  // Grab the key state and invert it, so that 1 = pressed
  gCurrentKeyState = ~REG_KEYINPUT & KEY_MASK;
}

/**
 * Get state of all keys at the last poll
 */
static inline u32 input_current_state() {
  return gCurrentKeyState;
}

/**
 * Get state of all keys at the previous poll
 */
static inline u32 input_previous_state() {
  return gPreviousKeyState;
}

/**
 * Returns > 0 if the key is pressed
 */
static inline u32 input_down(u32 key) {
  return gCurrentKeyState & key;
}

/**
 * Returns > 0 if the key is not pressed
 */
static inline u32 input_up(u32 key) {
  return ~gCurrentKeyState & key;
}

/**
 * Returns > 0 if the key was pressed last frame
 */
static inline u32 input_prev_down(u32 key) {
  return gPreviousKeyState & key;
}

/**
 * Returns > 0 if the key was not pressed last frame
 */
static inline u32 input_prev_up(u32 key) {
  return ~gPreviousKeyState & key;
}

/**
 * Returns > 0 if the key is pressed now and last poll
 */
static inline u32 input_held(u32 key) {
  return (gCurrentKeyState & gPreviousKeyState) & key;
}

/**
 * Returns > 0 if the key is pressed but wasn't last poll
 */
static inline u32 input_pressed(u32 key) {
  return (gCurrentKeyState &~ gPreviousKeyState) & key;
}

/**
 * Returns > 0 if the key isn't pressed but was last poll
 */
static inline u32 input_released(u32 key) {
  return (~gCurrentKeyState & gPreviousKeyState) & key;
}

