#pragma once

#include "tonc_types.h"

/**
 * Interrupt handling routines. Similar to Tonc implementation.
 * Only one ISR per interrupt is supported.
 *
 * @author Casey
 */

/**
 * IRQ index table from: http://www.coranac.com/tonc/text/interrupts.htm
 */
typedef enum eIRQIndex
{
    II_VBLANK = 0,
	II_HBLANK,
	II_VCOUNT,
	II_TIMER0,
    II_TIMER1,
	II_TIMER2,
	II_TIMER3,
	II_SERIAL,
    II_DMA0,
	II_DMA1,
	II_DMA2,
	II_DMA3,
    II_KEYPAD, 
	II_GAMEPAK,
	II_MAX
} eIRQIndex;

void irq_init();

/**
 * Enable an interrupt and set an ISR to handle it.
 * @param irqIndex - index of the IRQ
 * @param fnptr - function pointer to isr
 */
void irq_enable(eIRQIndex irqIndex, fnptr isr);

/**
 * Disable an interrpt
 * @param irqIndex - index of the IRQ
 */
void irq_disable(eIRQIndex irqIndex);
