#pragma once
#include "tonc_types.h"

//* Fixed point math functions 
//
//
//*/

#define FIXED_SHIFT 8
#define FIXED_SCALE 256

typedef s16 f16;
typedef s32 f32;

#include "lut.h"

typedef struct VECTOR {
	f32 x;
	f32 y;
	f32 z;
} VECTOR;

static inline VECTOR vector_add(VECTOR* v1, VECTOR* v2) {
	VECTOR return_vec;
	return_vec.x = v1->x + v2->x;
	return_vec.y = v1->y + v2->y;
	return_vec.z = v1->z + v2->z;
	return return_vec;
}

static inline VECTOR vector_sub(VECTOR* v1, VECTOR* v2) {
	VECTOR return_vec;
	return_vec.x = v1->x - v2->x;
	return_vec.y = v1->y - v2->y;
	return_vec.z = v1->z - v2->z;
	return return_vec;
}


static inline f16 s16_to_f16(s16 i) {
	return (f16) i << FIXED_SHIFT;
}

static inline u16 f16_to_u16(f16 i) {
	return (u16) i >> FIXED_SHIFT;
}

static inline s16 f16_to_s16(f16 i) {
	return (u16) i / FIXED_SCALE;
}

static inline f16 f_mul(f16 f1, f16 f2) {
	return (f1*f2) >> FIXED_SHIFT;
}

static inline f16 f_div(f16 f1, f16 f2) {
	return (f1*FIXED_SCALE)/f2;
}

static inline f32 s32_to_f32(s32 i) {
	return (f32) i << FIXED_SHIFT;
}

static inline u16 f32_to_u16(f32 i) {
	return (u16) i >> FIXED_SHIFT;
}

static inline u32 f32_to_u32(f32 i) {
	return (u32) i >> FIXED_SHIFT;
}

static inline s32 f32_to_s32(f32 i) {
	return (u16) i / FIXED_SCALE;
}

static inline f32 f_mul32(f32 f1, f32 f2) {
	return (f1*f2) >> FIXED_SHIFT;
}

static inline f32 f_div32(f32 f1, f32 f2) {
	return (f1*FIXED_SCALE)/f2;
}

static inline u32 lut_div(u32 in) {
	return DIV_LUT[in];
}

/*
Sin and cos for fixed points using luts
takes angle in degrees betweeb 0 and 359, will clamp to range
*/

static inline f16 sinf16(s16 angle) {
	
	if (angle < 0) {
		angle = 0;
	}

	if (angle > 359) {
		angle = 359;
	}

	return (f16) SIN_LUT[angle];
}

static inline f16 cosf16(s16 angle) {
	if (angle < 0) {
		angle = 0;
	}

	if (angle > 359) {
		angle = 359;
	}

	return (f16) COS_LUT[angle];

}

/**
* Return an angle in the range of 0-,359 from an angle in -359,-359
*
*/

static inline s16 abs_angle(s16 angle) {
	if (angle < 0) {
		return 360 + angle;
	}
	return angle;
}


static inline VECTOR vector_rotate_y(VECTOR* v, u16 angle) {
	VECTOR return_vec;
	angle = abs_angle(angle);
	f32 m1 = cosf16(angle);
	f32 m2 = -sinf16(angle);
	f32 m3 = -m2;
	f32 m4 = m1;

	return_vec.x = f_mul32(v->x, m1) + f_mul32(v->z, m3);
	return_vec.y = v->y;
	return_vec.z = f_mul32(v->x, m2) + f_mul32(v->z, m4); 
	return return_vec;
}

static inline f32 vector_sqr_mag(VECTOR* v) {
	return f_mul32(v->x, v->x) + f_mul32(v->y, v->y) + f_mul32(v->z, v->z); 
}

static inline VECTOR vector_mul(VECTOR* v, f16 s) {
	VECTOR ret_vector;
	ret_vector.x = f_mul32(v->x, s); 
	ret_vector.y = f_mul32(v->y, s); 
	ret_vector.z = f_mul32(v->z, s); 
	return ret_vector;
}

static inline VECTOR vector_negate(VECTOR* v) {
	VECTOR ret_vector;
	ret_vector.x = -v->x;
	ret_vector.y = -v->y;
	ret_vector.z = -v->z;
	return ret_vector;
}
/*
f16 Q_rsqrt(f16 number) {
	long i;
	f16 x2, y;
	const f16 threehalfs = (1<<8) | 128;
 
	x2 = number * 0.5F;
	y  = number;
	i  = * ( long * ) &y;                       // evil floating point bit level hacking
	i  = 0x5f3759df - ( i >> 1 );               // what the fuck?
	y  = * ( float * ) &i;
	y  = y * ( threehalfs - ( x2 * y * y ) );   // 1st iteration
 
	return y;
}*/

/**
 * Compute the affine matrix given the source params.
 * 
 * TODO: Move to C file
 *
 * @param out param for the result
 * @param src source params
 */

 /*
void affine_rotscale(BG_AFFINE* out, const AFF_SRC_EX* src) {

	f32 pa;
	f32 pb;
	f32 pc;
	f32 pd;

	// TODO: Lookup in lut using src->alpha
	f16 sina = sinf16(src->alpha);
	f16 cosa = cosf16(src->alpha);

	pa = f_mul32(src->sx, cosa);
	pb = f_mul32(-src->sx, sina);
	pc = f_mul32(src->sy, sina);
	pd = f_mul32(src->sy, cosa);

	out->pa = pa;
	out->pb = pb;
	out->pc = pc;
	out->pd = pd;
	
	out->dx = src->tex_x - (pa * src->scr_x + pb * src->scr_y);
	out->dy = src->tex_y - (pc * src->scr_x + pd * src->scr_y);

}*/
