#pragma once

/**
 * Sprite functions
 *
 * @author Stephan
 */


/**
 * Initialize OAM to decent initial values
 * This is required because otherwise all sprites will be top left corner (emulator)
 * or random memory? (device, unless it zeros RAM)
 *
 * NOTE: Must be called during VBLANK because it modifies OAM
 * Does not modify REG_DISPCNT to turn on sprites, so you must do that still
 */
void init_sprites();

/**
 * Write sprite attributes directly to OAM
 *
 * This is ok for modifying a few sprites during VBLANK but
 * a solution that would work oustude of vblank would be to 
 * modify copies in main memory and then kick off a copy into
 * OAM at the start of VBLANK
 *
 * @param id - which of the 128 sprites to mpdify, 0 - 127
 * @param xpos, ypos - the pixel coordinated of the sprite's top left corner 
 * @param sprite_number - which tile number in sprite VRAM to use for the sprite
 *
 * NOTE: Must be called during VBLANK because it modifies OAM
 */
void sprite_set(u32 id, u32 xpos, u32 ypos, u16 sprite_number, 
				u32 attr0_flags, u32 attr1_flags, u32 attr2_flags);

void rotate_sprite_group(u32 id, s32 angle);

//This struct is 64 bits, so it shouldn't require any GCC
//alignment flags to work properly
typedef struct Sprite {
	u16 attr0;
	u16 attr1;
	u16 attr2;
	s16 attr3;
} ALIGN4 Sprite;

typedef struct SpriteAffine {
	u16 fill0[3];
    s16 pa;
    u16 fill1[3];
    s16 pb;
    u16 fill2[3];
    s16 pc;
    u16 fill3[3];
    s16 pd;
} ALIGN4 SpriteAffine;
