#!/usr/bin/env python2.7

import os
import glob
import struct
import argparse

IDENTIFIER_UPPER = 0xd0c6ed93
IDENTIFIER_LOWER = 0x9a04ffa3
SEARCH_SIZE      = 512

if __name__ == "__main__":
	parser = argparse.ArgumentParser(
		description='Generate and append a filesystem to a gba image.')
	parser.add_argument('gba_file',
						help='gba file to append filesystem to')
	parser.add_argument('assets_folder',
						help='folder of assets to add to the filesystem')
	parser.add_argument('out_file',
						help='output filename for the gba + filesystem')

	args = parser.parse_args()
		
	gba_file = open(args.gba_file)
	gba_bytes = gba_file.read()

	gba_file.close()
	
	print "Read GBA file: %s, %s bytes" % (args.gba_file, len(gba_bytes))
	print "Generating filesystem..."

	assets = glob.glob(args.assets_folder + '/*')

	filesystem_bytes = bytearray()

	# Generate FILESYSTEM_FILE structs and pack all file data
	
	offset = 0	
	for asset in assets:
		asset_file = open(asset)
		asset_bytes = asset_file.read()
		asset_length = len(asset_bytes)
		base_name = os.path.basename(asset)

		if len(base_name) > 31:
			print "  Filename %s is too long" % base_name
			exit(1)

		print "  Adding %s, %s bytes at offset %s" % (base_name, len(asset_bytes), offset)
					
		# Create FILESYSTEM_FILE struct
		header = struct.pack("32sI", base_name, asset_length)
		
		filesystem_bytes.extend(header)
		filesystem_bytes.extend(asset_bytes)

		offset += len(header)
		offset += len(asset_bytes)
		
		if (asset_length % 4) != 0:
			padding = bytearray(b'\x00' * (-(asset_length % 4) + 4))
			filesystem_bytes.extend(padding)
			offset += len(padding)

	filesystem_start = (len(gba_bytes) / SEARCH_SIZE) * SEARCH_SIZE
		
	if (filesystem_start % len(gba_bytes)) != 0:
		filesystem_start += SEARCH_SIZE

	# Construct final output
		
	output_bytes = bytearray()

	# Generate header FILESYSTEM_INFO struct

	filesystem_info = struct.pack(
		"IIII",
		IDENTIFIER_UPPER,
		IDENTIFIER_LOWER,

		# Filesytem length + this header
		len(filesystem_bytes) + 16,
		len(assets))

	output_bytes.extend(gba_bytes)
		
	end_padding_length = filesystem_start - len(gba_bytes)
	if end_padding_length != 0:
		output_bytes.extend(bytearray(b'\x00' * end_padding_length))

	output_bytes.extend(filesystem_info)
	output_bytes.extend(filesystem_bytes)
	
	print "Writing filesystem at offset %s" % filesystem_start

	#print repr(output_bytes)
	
	output_file = open(args.out_file, 'w')
	output_file.write(output_bytes)
	output_file.close()
	exit(0)
	
